package com.jpattern.gwt.client.event;

import java.util.List;

import com.jpattern.gwt.client.ApplicationProxy;
import com.jpattern.gwt.client.IApplicationProvider;
import com.jpattern.gwt.client.command.ACommand;
import com.jpattern.gwt.client.command.ICommandCallBack;
import com.jpattern.gwt.client.logger.ILogger;
import com.jpattern.gwt.client.presenter.IPresenter;
import com.jpattern.shared.result.IErrorMessage;
import com.jpattern.shared.result.IResult;

/**
 * 
 * @author Francesco Cina'
 *
 * 14 Apr 2011
 */
public abstract class AEvent<T> implements IEvent<T> {
	
	private final IPresenter presenter;
	private IApplicationProvider provider;
	private IEventCallback<T> eventCallback;
	private final ILogger logger;

	public AEvent(IPresenter presenter) {
		this(presenter, ApplicationProxy.getInstance().getApplicationProvider());
	}
	
	public AEvent(IPresenter presenter, IApplicationProvider provider) {
		this.presenter = presenter;
		this.provider = provider;
		this.logger = getProvider().getLoggerService().getLogger(this.getClass());
	}

	/* (non-Javadoc)
	 * @see com.jpattern.gwt.event.IEvent#launch()
	 */
	@Override
	public final void launch(IEventCallback<T> eventCallback) {
		this.eventCallback = eventCallback;
		logger.debug("launch", "start execution");
		presenter.onEventStart();
		ACommand command = exec();
		command.visit(getProvider());
		command.exec( new EventCommandCallback());
	}

	/* (non-Javadoc)
	 * @see com.jpattern.gwt.event.IEvent#callback(com.jpattern.gwt.command.IWebResult)
	 */
	private void callback(IResult commandResult) {
		IEventResult<T> eventResult = afertExec(commandResult);
		logger.debug("callback", "start execution (event execution finished)");
		presenter.onEventEnd();
		if ( eventResult.getErrorMessages().isEmpty() ) {
			logger.debug("callback", "calling event callback");
			eventCallback.callback(eventResult);
		}
		else {
			logger.warn("callback", eventResult.getErrorMessages().size() + " errors appeared");
			presenter.onEventError(eventResult.getErrorMessages());
			try {
				logger.debug("callback", "calling event callback");
				eventCallback.callback(eventResult);
			} finally {
				checkManagedErrors(eventResult.getErrorMessages());
			}
		}
	}
	
	protected abstract ACommand exec();
	
	protected abstract IEventResult<T> afertExec(IResult webResult);

	protected IApplicationProvider getProvider() {
		if (provider == null) {
			provider = ApplicationProxy.getInstance().getApplicationProvider();
		}
		return provider;
	}
	
	private void checkManagedErrors(List<IErrorMessage> list) {
		for ( IErrorMessage errorMessage : list ) {
			getProvider().getManagedErrorService().launch( errorMessage, presenter, this, eventCallback );		
		}
	}
	
	private class EventCommandCallback implements ICommandCallBack {

		@Override
		public void callback(IResult commandResult) {
			AEvent.this.callback(commandResult);			
		}
		
	}
}
